//-----------------------------------------------------------
//
//-----------------------------------------------------------
class XComTacticalInput_FreeCameraRotation extends XComTacticalInput dependson(X2GameRuleset) config(TacticalInput_FreeCameraRotation);

var config float CAMERA_ROTATION_DEG_FIXED;
var config float CAMERA_ROTATION_DEG_PER_SECOND;
var config float CAMERA_ZOOM_PER_SECOND;

var bool freeRotationActive, freeZoomActive;
var float lastPressed;
var bool bFreeLook;

exec function ToggleFreeLook()
{
	bFreeLook = !bFreeLook;
}

function bool CheckAndRotateCamera( int ActionMask, float direction )
{
	local float timeDelta;

	if ( ( ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_PRESS ) != 0 )
	{
		freeRotationActive = false;
		lastPressed = 0.0f;
		return true;
	}
	else if ( ( ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_POSTHOLD_REPEAT ) != 0 )
	{
		freeRotationActive = true;
		timeDelta = (lastPressed > 0.0f) ? (WorldInfo.TimeSeconds - lastPressed) : 0.0f;
		lastPressed = WorldInfo.TimeSeconds;
		XComTacticalController(Outer).YawCamera(CAMERA_ROTATION_DEG_PER_SECOND * direction * timeDelta);
		return true;
	}
	else if ( ( ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_RELEASE ) != 0 )
	{
		if( !freeRotationActive )
			XComTacticalController(Outer).YawCamera(CAMERA_ROTATION_DEG_FIXED * direction);
		freeRotationActive = false;
		return true;
	}

	return false;
}

function bool CheckAndRotateCameraRight( int ActionMask )
{
	return CheckAndRotateCamera( ActionMask, -1 );
}

function bool CheckAndRotateCameraLeft( int ActionMask )
{
	return CheckAndRotateCamera( ActionMask, 1 );
}

function bool CheckAndZoomCamera( int ActionMask, bool bZoomIn )
{
	local float timeDelta;

	if ( ( ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_PRESS ) != 0 )
	{
		freeZoomActive = false;
		lastPressed = 0.0f;
		return true;
	}
	else if ( ( ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_POSTHOLD_REPEAT ) != 0 )
	{
		freeZoomActive = true;
		timeDelta = (lastPressed > 0.0f) ? (WorldInfo.TimeSeconds - lastPressed) : 0.0f;
		lastPressed = WorldInfo.TimeSeconds;
		XComPresentationLayer(XComTacticalController(Outer).Pres).ZoomCameraScroll( bZoomIn, CAMERA_ZOOM_PER_SECOND * timeDelta );
		return true;
	}
	else if ( ( ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_RELEASE ) != 0 )
	{
		if( !freeZoomActive )
			XComPresentationLayer(XComTacticalController(Outer).Pres).ZoomCameraScroll( bZoomIn );
		freeZoomActive = false;
		return true;
	}

	return false;
}

state InReplayPlayback
{
	function bool DPad_Right( int ActionMask )
	{
		return CheckAndRotateCameraRight( ActionMask );
	}

	function bool DPad_Left( int ActionMask )
	{
		return CheckAndRotateCameraLeft( ActionMask );
	}
}

state UsingTargetingMethod
{
	function bool DPad_Right( int ActionMask )
	{
		return CheckAndRotateCameraRight( ActionMask );
	}

	function bool DPad_Left( int ActionMask )
	{
		return CheckAndRotateCameraLeft( ActionMask );
	}
}

state Multiplayer_Inactive
{
	function bool DPad_Right( int ActionMask )
	{
		return CheckAndRotateCameraRight( ActionMask );
	}
	function bool DPad_Left( int ActionMask )
	{
		return CheckAndRotateCameraLeft( ActionMask );
	}
}

state ActiveUnit_Moving
{
	function bool DPad_Right( int ActionMask )
	{
		return CheckAndRotateCameraRight( ActionMask );
	}
	function bool DPad_Left( int ActionMask )
	{
		return CheckAndRotateCameraLeft( ActionMask );
	}
	function bool Key_T( int ActionMask )
	{
		return CheckAndZoomCamera( ActionMask, false );
	}
	function bool Key_G( int ActionMask )
	{
		return CheckAndZoomCamera( ActionMask, true );
	}
	function bool MMouse(int ActionMask)
	{
		if( (ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_PRESS) != 0 )
		{
			if(bFreeLook || `CHEATMGR.bAllowFancyCameraStuff)
			{
				m_bMouseFreeLook = true;
			}
			else
			{
				XComPresentationLayer(XComTacticalController(Outer).Pres).ZoomCameraOut();
			}
		}
		else if( (ActionMask & class'UIUtilities_Input'.const.FXS_ACTION_RELEASE) != 0 )
		{
			if(!m_bMouseFreeLook)
			{
				XComPresentationLayer(XComTacticalController(Outer).Pres).ZoomCameraIn();
			}
			m_bMouseFreeLook = false;
		}

		return true;
	}
}